window.addEventListener('DOMContentLoaded', async event => {
    // window.console.assert(window.document.readyState !== 'loading');
    // window.console.info("'window' object properties: " + Object.getOwnPropertyNames(globalThis).join("-"));
    // ES module technology only:
    // window.console.info("JavaScript executable file: " + import.meta.url);
    //
    // window.fetch("./build/Barcode.wasm").then(bytes => bytes.arrayBuffer()).then(async buffer => {
    //     window.console.assert(buffer instanceof ArrayBuffer);
    //     const module = await WebAssembly.compile(buffer);
    //     window.console.assert(module instanceof WebAssembly.Module);
    //     // Etc.
    // });

    /** Compilation is such that 'abort' is bypassed:
     * 'asc assembly/Barcode.ts --target release --use abort=assembly/Barcode/Abort'
     * As a result, AssemblyScript compiler no longer imports 'abort'... (https://www.assemblyscript.org/concepts.html#special-imports),
     * but 'console.info(checksum.toString());' requires binding:
     */
    const import_ = {env: {"console.info": data => console.info(`'console.info': ${data}`)}};

    // 'import_' is *REQUIRED* in 'WebAssembly.instantiateStreaming' only if the instantiated module actually imports something.
    // https://stackoverflow.com/questions/48039547/webassembly-typeerror-webassembly-instantiation-imports-argument-must-be-pres
    WebAssembly.instantiateStreaming(window.fetch("./build/Barcode.wasm"), import_).then(web_assembly => {
        window.console.assert(web_assembly.module instanceof window.WebAssembly.Module);
        window.console.assert(web_assembly.instance instanceof window.WebAssembly.Instance);

        // Memory provided by WebAssembly module (https://developer.mozilla.org/en-US/docs/WebAssembly/JavaScript_interface/Memory):
        const memory = web_assembly.instance.exports.memory;
        window.console.assert(memory instanceof WebAssembly.Memory && memory.buffer instanceof ArrayBuffer);
        // Alter 'memory' ('ArrayBuffer' objects cannot be directly handled):
        // const data = new DataView(memory.buffer);

        const imports = WebAssembly.Module.imports(web_assembly.module); // Array of declared imports...
        for (const import_ of imports)
            window.console.log(JSON.stringify(import_)); // 'imports[0]': '{"module":"env","name":"console.info","kind":"function"}'

        window.console.assert("Barcode_checksum" in web_assembly.instance.exports);
        window.document.body.innerText = web_assembly.instance.exports.Barcode_checksum(4719512002889n); // Display '0' meaning "OK"...
    }, error => {
        window.console.error(error); // window.alert(error);
    });
});

// (import "a" "f" (func ...))
// (import "a" "g" (func ...))
// (import "b" "m" (memory 0))
// {
//     a: {f: function (...) { ... }, g: function (...) {...}},
//     b: {m: new WebAssembly.Memory(...)}
// }
// const import_ = {env: {abort: (message, fileName, lineNumber, columnNumber) => {window.console.error(message); /* Etc. */}}};
// WebAssembly.instantiate(module, import_).then(executable => { // Etc.







